import collections
from django.core.cache import get_cache


class DjangoCache(collections.MutableMapping):
    """Cache that uses the Django Cache Backend"""

    def __init__(self, backend='default', prefix='mxr:', ttl=604800):
        """
        Initalize the cache

        Arguments:
        cache -- the name of the Django cache to use (if not the default cache)
        prefix -- the prefix to use when setting mx record keys (defaults to 'mxr:')
        ttl -- the timeout for mx record caches in seconds (defaults to 1 week)

        """
        self.prefix = prefix
        self.ttl = ttl
        self.cache = get_cache(backend)

    def __getitem__(self, key):
        """Get an item from the cache"""
        return self.cache.get(self.__keytransform__(key))

    def __setitem__(self, key, value):
        """Set a cache item"""
        return self.cache.set(self.__keytransform__(key), value, self.ttl)

    def __delitem__(self, key):
        """Delete an item from the cache"""
        self.cache.delete(self.__keytransform__(key))

    def __keytransform__(self, key):
        """Generate the key name"""
        return ''.join([self.prefix, str(key)])

    def __value_generator__(self, keys):
        """Generator to pull multiple keys"""
        for key in keys:
            yield self.cache.get(key)

    def __iter__(self):
        """Cache Iterator (not available with Django's cache)"""
        return iter([])

    def __len__(self):
        """Cache Length (not available with Django's cache)"""
        return 0



from django.core.exceptions import ValidationError
from django.core.validators import EmailValidator
from django.utils.translation import string_concat, ugettext_lazy as _
from flanker.addresslib import address, validate


class FlankerValidator(EmailValidator):
    suggest_fragment = _('(Did you mean: "%(suggestion)s"?)')

    def __init__(self, suggest_fragment=None, **kwargs):
        super(FlankerValidator, self).__init__(**kwargs)
        if suggest_fragment is not None:
            self.suggest_fragment = suggest_fragment

    def __call__(self, value):
        super(FlankerValidator, self).__call__(value)

        if address.validate_address(value) is None:
            suggestion = validate.suggest_alternate(value)
            if suggestion:
                message = string_concat(self.message, ' ', self.suggest_fragment)
                raise ValidationError(message, code=self.code, params={
                    'suggestion': suggestion
                })
            raise ValidationError(self.message, code=self.code)


flanker_validator = FlankerValidator()



from django.conf import settings
from django.db import models
from flanker import addresslib
from . import driver, forms, validators


if getattr(settings, 'FLANKER_DRIVER_ENABLED', True):
    params = getattr(settings, 'FLANKER_DRIVER_PARAMS', {})
    addresslib.set_mx_cache(driver.DjangoCache(**params))


class EmailField(models.EmailField):
    default_validators = [validators.flanker_validator]

    def formfield(self, **kwargs):
        defaults = {
            'form_class': forms.EmailField,
        }
        defaults.update(kwargs)
        return super(EmailField, self).formfield(**defaults)



#!/usr/bin/env python
"""
django-flanker
==============

|travis-ci|_ |coveralls|_ |downloads|_

Django-flanker provides a drop-in replacement EmailFields for Django forms and
models that uses Mailgun's `flanker <https://github.com/mailgun/flanker>`_
library for more extensive validation.

.. |travis-ci| image:: https://secure.travis-ci.org/dmpayton/django-flanker.png
.. _travis-ci: http://travis-ci.org/dmpayton/django-flanker

.. |coveralls| image:: https://coveralls.io/repos/dmpayton/django-flanker/badge.png
.. _coveralls: https://coveralls.io/r/dmpayton/django-flanker

.. |downloads| image:: https://pypip.in/d/django-flanker/badge.png
.. _downloads: https://pypi.python.org/pypi/django-flanker/

"""

from django_flanker import __version__, __description__, __license__

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

setup(
    name='django-flanker',
    version=__version__,
    description=__description__,
    long_description=__doc__,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
    keywords='django flanker mailgun email validation',
    maintainer='Derek Payton',
    maintainer_email='derek.payton@gmail.com',
    url='https://github.com/dmpayton/django-flanker',
    download_url='https://github.com/dmpayton/django-flanker/tarball/v%s' % __version__,
    license=__license__,
    include_package_data=True,
    packages=find_packages(exclude=['tests']),
    install_requires=[
        'django',
        'flanker'
    ],
    zip_safe=False,
    )



__author__ = 'Derek Payton <derek.payton@gmail.com>'
__copyright__ = 'Copyright (c) Derek Payton'
__description__ = 'Extended EmailField validation using flanker'
__version__ = '0.2.0'
__license__ = 'MIT License'
